package pwc.taxtech.atms.service;

import java.util.List;
import java.util.Map;

import pwc.taxtech.atms.dpo.UserOrgRoleDto;
import pwc.taxtech.atms.dpo.UserRoleInfo;
import pwc.taxtech.atms.dpo.DimensionValueOrgDto;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.dimension.DimensionValueUpdateDto;
import pwc.taxtech.atms.dto.navtree.NavTreeDto;
import pwc.taxtech.atms.dpo.OrganizationDto;
import pwc.taxtech.atms.dto.role.RoleDisplayDto;
import pwc.taxtech.atms.dto.role.RoleDto;
import pwc.taxtech.atms.dto.role.RoleDtoTree;
import pwc.taxtech.atms.dto.role.RolePermissionDto;
import pwc.taxtech.atms.dto.role.UpdateRoleInfo;
import pwc.taxtech.atms.dto.user.DimensionRole;
import pwc.taxtech.atms.dto.user.DimensionUser;
import pwc.taxtech.atms.dpo.UserOrgDto;
import pwc.taxtech.atms.dto.user.UserRoleDimensionValueDto;
import pwc.taxtech.atms.dto.user.UserRoleDto;
import pwc.taxtech.atms.dto.user.UserRoleUpdateDto;
import pwc.taxtech.atms.dto.user.VMUser;
import pwc.taxtech.atms.entity.UserDimensionValue;
import pwc.taxtech.atms.entity.UserDimensionValueOrg;
import pwc.taxtech.atms.entity.UserOrganization;

public interface RoleService {

    List<NavTreeDto> getRoleTreeList();

    List<UserRoleInfo> getUserRoleByUser(VMUser vmUser);

    List<UserRoleInfo> getAllOwnUserRoleList();

    List<UserRoleInfo> getAllUserRoleList();

    List<UserRoleDto> getAllRoleListByUserId(String userId, String serviceTypeId);

    List<UserOrgDto> getUserByDimensionValue(List<OrganizationDto> orgList,
                                             List<UserDimensionValue> userDimensionValueList, List<UserDimensionValueOrg> userDimensionValueOrgList,
                                             List<UserOrganization> userOrganizationList, List<UserOrgRoleDto> originalRoleList,
                                             List<UserOrgRoleDto> userDimensionValueRoleList, List<UserOrgRoleDto> userOrganizationRoleList,
                                             String dimensionId, String dimensionValueId);

    List<RoleDtoTree> getRoleList();
    
    List<UserRoleDto> getUsersByRoleId(String roleId);
    
    /**
     * 查询某角色额外用户
     * @param roleId
     * @return
     */
    List<UserRoleDto> getExtraUsersByRoleId(String roleId);
    
    /**
     * 为角色添加用户
     * @param roleId
     * @param serviceTypeId
     * @param userIdList
     */
    void addUsersToRole(String roleId, String serviceTypeId, List<String> userIdList);
    
    void removeUserRole(String userId, List<String> roleIdList, String serviceTypeId);
    
    /**
     * Get role list by roleTypeId
     * @param roleTypeId
     * @return List<RoleDto>
     */
    List<RoleDto> getRoleListByRoleType(String roleTypeId);
    
    /**
     * 增加角色分类
     * @param roleCategoryName
     * @param roleCategoryId
     */
    void addRoleCategory(String roleCategoryName, String roleCategoryId);
    
    /**
     * 更新角色分类名称
     * @param roleCategoryName
     * @param roleCategoryId
     */
    void updateRoleCategory(String roleCategoryName, String roleCategoryId);

    /**
     * 删除角色分类
     * @param roleCategoryId
     * @return OperationResultDto
     */
    @SuppressWarnings("rawtypes")
    OperationResultDto deleteRoleCategory(String roleCategoryId);
    
    /**
     * Add a new role info
     * @param RoleDisplayDto roleDto
     * @return roleId
     */
    String addRole(RoleDisplayDto roleDto);
    
    /**
     * Update a role info
     * @param updateRole
     * @param roleId
     */
    void updateRole(UpdateRoleInfo updateRole, String roleId);
    
    /**
     * validate if the rolename exists
     * @param roleName
     * @param oldRoleName
     * @return true - not exist; false - exist
     */
    boolean validateRoleNameUnique(String roleName, String oldRoleName);
    
    /**
     * Delete a role info
     * @param roleDto
     */
    void deleteRole(RoleDto roleDto);
    
    /**
     * check mapping between user and role
     * @param roleId
     * @return
     */
    boolean checkUserRole(String roleId);
    
    /**
     * @param model
     * @return
     */
    List<UserRoleInfo> getUserRoleList(UserRoleDimensionValueDto userRoleDimensionValueDto);
    
    /**
     * 根据serviceType获取所有角色权限list
     * @param serviceTypeId
     * @return List<RolePermissionDto>
     */
    List<RolePermissionDto> getAllRolePermission(String serviceTypeId);
    
    /**
     * 更新用户角色列表
     * @param userRoleUpdateDto
     */
    void updateUserRole(UserRoleUpdateDto userRoleUpdateDto);
    
    /**
     * 用户详情页面，增加范围
     * @param dimensionValueUpdateDto
     */
    void updateDimensionValues(DimensionValueUpdateDto dimensionValueUpdateDto);
    
    /**
     * 用户详情页面, 增加机构
     * @param orgIdList
     * @param userId
     */
    void updateUserOrg(List<String> orgIdList, String userId);
    
    /**
     * 获取维度，角色，用户列表
     * @param dimensionId
     * @param dimensionValueId
     */
    List<DimensionRole> getDimensionRoleUserList(String dimensionId, String dimensionValueId);

    /**
     * 获取某一个维度的 用户 和 机构列表，角色列表
     * @param dimensionId
     * @param dimensionValueId
     */
    List<DimensionUser> getDimensionUserRoleList(String dimensionId, String dimensionValueId);

    Map<String, Integer> getActiveUserRoleList(List<OrganizationDto> orgDtoList,
            List<DimensionValueOrgDto> dimensionValueOrgList, String areaid);
    
    List<UserRoleInfo> getActiveUserRoleListByAreaId(String areaId);
}
