package pwc.taxtech.atms.vat.service.impl;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import pwc.taxtech.atms.common.util.MyAsserts;
import pwc.taxtech.atms.constant.enums.EnumServiceType;
import pwc.taxtech.atms.dao.FormulaAdminMapper;
import pwc.taxtech.atms.dpo.CellTemplatePerGroupDto;
import pwc.taxtech.atms.dpo.GroupId;
import pwc.taxtech.atms.entity.ProjectServiceType;
import pwc.taxtech.atms.entity.ProjectServiceTypeExample;
import pwc.taxtech.atms.exception.Exceptions;
import pwc.taxtech.atms.vat.dao.CellDataMapper;
import pwc.taxtech.atms.vat.dao.ReportMapper;
import pwc.taxtech.atms.vat.entity.CellData;
import pwc.taxtech.atms.vat.entity.CellDataExample;
import pwc.taxtech.atms.vat.entity.Report;
import pwc.taxtech.atms.vat.entity.ReportExample;

import java.util.List;

import static pwc.taxtech.atms.constant.Constant.FIRST_OR_DEFAULT;

@Component
public class FormulaAgent extends VatAbstractService {
    @Autowired
    public FormulaAdminMapper adminMp;
    @Autowired
    public ReportMapper reportMapper;
    @Autowired
    public CellDataMapper cellDataMapper;

    public List<CellTemplatePerGroupDto> getCellTemplateGroupDto(Long templateGroupId, String projectId) {
        ProjectServiceTypeExample pst = new ProjectServiceTypeExample();
        pst.createCriteria().andTemplateGroupIdEqualTo(Long.valueOf(templateGroupId))
                .andServiceTypeIdEqualTo(EnumServiceType.VAT.getCode() + "");
        List<ProjectServiceType> pstReult = projectServiceTypeMapper.selectByExample(pst);

        List<CellTemplatePerGroupDto> cellTemplates = adminMp.getCellTemplatePerGroupDto(templateGroupId);
        if (pstReult == null || pstReult.isEmpty()) {
            List<GroupId> groupIds = adminMp.getTemplateGroupId(projectId);
            GroupId groupId;

            if (groupIds.size() > 0) {
                groupId = groupIds.stream().filter(m -> m.isDefault != 1).findFirst().get();
                if (groupId == null) groupId = groupIds.get(0);
                cellTemplates.addAll(adminMp.getCellTemplatePerGroupDto(groupId.groupId));
            }
        }
        return cellTemplates;
    }

    private Report getReportByTemplate(String templateId, Integer periodId) {
        ReportExample example = new ReportExample();
        example.createCriteria().andTemplateIdEqualTo(Long.valueOf(templateId))
                .andPeriodEqualTo(periodId);
        List<Report> reports = reportMapper.selectByExample(example);

        if (reports != null && !reports.isEmpty()) return reports.get(FIRST_OR_DEFAULT);
        return null;
    }

    private CellData getCellDataListByTemplate(String templateId, Long reportId) {
        CellDataExample dataExample = new CellDataExample();
        dataExample.createCriteria().andCellTemplateIdEqualTo(Long.valueOf(templateId)).andReportIdEqualTo(reportId);
        List<CellData> cellDataList = cellDataMapper.selectByExample(dataExample);

        if (cellDataList != null && !cellDataList.isEmpty()) return cellDataList.get(FIRST_OR_DEFAULT);
        return null;
    }

    public String getPastProjectId(int year, String orgId) {
        return adminMp.getPastProjectId(year, orgId);
    }

    public CellData getCellData(String templateId, String cellId, int periodId) {
        Report report = getReportByTemplate(templateId, periodId);
        MyAsserts.assertNotNull(report, Exceptions.BB_REPORT_NULL);

        CellData cellData = getCellDataListByTemplate(cellId, report.getId());
        MyAsserts.assertNotNull(cellData, Exceptions.BB_CELL_DATA_NULL);
        return cellData;
    }

}
